"""init

Revision ID: 635350d66cc8
Revises: 
Create Date: 2017-04-18 21:25:00.509894

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '635350d66cc8'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('person',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.PrimaryKeyConstraint('id')
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('person')
    # ### end Alembic commands ###
